@@ -59,6 +59,8 @@
 #define DEFAULT_HEIGHT 200
 #define DEFAULT_TITLE  "GtkGLExt Template"
 
+#define TIMEOUT_INTERVAL 10
+
 
 /**************************************************************************
  * Global variable declarations.
@@ -71,8 +73,8 @@ static gboolean animate = FALSE;
  * The following section contains the function prototype declarations.
  **************************************************************************/
 
-static void         idle_add          (GtkWidget   *widget);
-static void         idle_remove       (GtkWidget   *widget);
+static void         timeout_add       (GtkWidget   *widget);
+static void         timeout_remove    (GtkWidget   *widget);
 
 static void         toggle_animation  (GtkWidget   *widget);
 
@@ -162,6 +164,8 @@ expose_event (GtkWidget      *widget,
   if (!gdk_gl_drawable_gl_begin (gldrawable, glcontext))
     return FALSE;
 
+  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+
   /*** Fill in the details here. ***/
 
   /* Swap buffers */
@@ -177,18 +181,22 @@ expose_event (GtkWidget      *widget,
 }
 
 /***
- *** The idle function. Often in animations,
- *** idle functions are suitable for continous
+ *** The timeout function. Often in animations,
+ *** timeout functions are suitable for continous
  *** frame updates.
  ***/
 static gboolean
-idle (GtkWidget *widget)
+timeout (GtkWidget *widget)
 {
   g_print (".");
 
   /*** Fill in the details here ***/
 
-  gtk_widget_queue_draw (widget);
+  /* Invalidate the whole window. */
+  gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
+
+  /* Update synchronously. */
+  gdk_window_process_updates (widget->window, FALSE);
 
   return TRUE;
 }
@@ -340,33 +348,33 @@ key_press_event (GtkWidget   *widget,
 
 
 /**************************************************************************
- * The following section contains the idle function management routines.
+ * The following section contains the timeout function management routines.
  **************************************************************************/
 
 /***
- *** Helper functions to add or remove the idle function.
+ *** Helper functions to add or remove the timeout function.
  ***/
 
-static guint idle_id = 0;
+static guint timeout_id = 0;
 
 static void
-idle_add (GtkWidget *widget)
+timeout_add (GtkWidget *widget)
 {
-  if (idle_id == 0)
+  if (timeout_id == 0)
     {
-      idle_id = gtk_idle_add_priority (GDK_PRIORITY_REDRAW,
-				       (GtkFunction) idle,
-				       widget);
+      timeout_id = gtk_timeout_add (TIMEOUT_INTERVAL,
+                                    (GtkFunction) timeout,
+                                    widget);
     }
 }
 
 static void
-idle_remove (GtkWidget *widget)
+timeout_remove (GtkWidget *widget)
 {
-  if (idle_id != 0)
+  if (timeout_id != 0)
     {
-      gtk_idle_remove (idle_id);
-      idle_id = 0;
+      gtk_timeout_remove (timeout_id);
+      timeout_id = 0;
     }
 }
 
@@ -381,7 +389,7 @@ map_event (GtkWidget *widget,
 {
   g_print ("%s: \"map_event\":\n", gtk_widget_get_name (widget));
   if (animate)
-    idle_add (widget);
+    timeout_add (widget);
 
   return TRUE;
 }
@@ -396,7 +404,7 @@ unmap_event (GtkWidget *widget,
 	     gpointer   data)
 {
   g_print ("%s: \"unmap_event\":\n", gtk_widget_get_name (widget));
-  idle_remove (widget);
+  timeout_remove (widget);
 
   return TRUE;
 }
@@ -414,9 +422,9 @@ visibility_notify_event (GtkWidget          *widget,
   if (animate)
     {
       if (event->state == GDK_VISIBILITY_FULLY_OBSCURED)
-	idle_remove (widget);
+	timeout_remove (widget);
       else
-	idle_add (widget);
+	timeout_add (widget);
     }
 
   return TRUE;
@@ -437,12 +445,12 @@ toggle_animation (GtkWidget *widget)
 
   if (animate)
     {
-      idle_add (widget);
+      timeout_add (widget);
     }
   else
     {
-      idle_remove (widget);
-      gtk_widget_queue_draw (widget);
+      timeout_remove (widget);
+      gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
     }
 }
 
@@ -499,10 +507,6 @@ create_window (GdkGLConfig *glconfig)
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (window), DEFAULT_TITLE);
 
-#ifndef G_OS_WIN32
-  /* Perform the resizes immediately */
-  gtk_container_set_resize_mode (GTK_CONTAINER (window), GTK_RESIZE_IMMEDIATE);
-#endif
   /* Get automatically redrawn if any of their children changed allocation. */
   gtk_container_set_reallocate_redraws (GTK_CONTAINER (window), TRUE);
 
@@ -557,7 +561,7 @@ create_window (GdkGLConfig *glconfig)
   g_signal_connect_swapped (G_OBJECT (window), "key_press_event",
 			    G_CALLBACK (key_press_event), drawing_area);
 
-  /* For idle function. */
+  /* For timeout function. */
   g_signal_connect (G_OBJECT (drawing_area), "map_event",
 		    G_CALLBACK (map_event), NULL);
   g_signal_connect (G_OBJECT (drawing_area), "unmap_event",
